@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import sys
 import urlparse
 
@@ -9,7 +9,7 @@ __all__ = ['name', 'get']
 
 
 def name():
-    return urlparse.urlparse(config.get('database', 'uri', '')).scheme
+    return urlparse.urlparse(config.get('database', 'uri', default='')).scheme
 
 
 def get(prop):
