@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import os
 import sys
 import logging
@@ -11,8 +11,8 @@ except ImportError:
 from lxml import etree
 from trytond.model import ModelView, ModelSQL, fields
 from trytond import backend
-from trytond.pyson import CONTEXT, Eval, Bool, PYSONDecoder
-from trytond.tools import safe_eval, file_open
+from trytond.pyson import Eval, Bool, PYSONDecoder
+from trytond.tools import file_open
 from trytond.transaction import Transaction
 from trytond.wizard import Wizard, StateView, Button
 from trytond.pool import Pool
@@ -24,6 +24,8 @@ __all__ = [
     'ViewTreeWidth', 'ViewTreeState', 'ViewSearch',
     ]
 
+logger = logging.getLogger(__name__)
+
 
 class View(ModelSQL, ModelView):
     "View"
@@ -137,11 +139,10 @@ class View(ModelSQL, ModelView):
                 rng_type = view.inherit.type if view.inherit else view.type
                 validator = etree.RelaxNG(etree=cls.get_rng(rng_type))
                 if not validator.validate(tree):
-                    logger = logging.getLogger('ir')
                     error_log = reduce(lambda x, y: str(x) + '\n' + str(y),
                             validator.error_log.filter_from_errors())
-                    logger.error('Invalid xml view:\n%s'
-                        % (str(error_log) + '\n' + xml))
+                    logger.error('Invalid xml view:\n%s',
+                        str(error_log) + '\n' + xml)
                     cls.raise_user_error('invalid_xml', (view.rec_name,))
             root_element = tree.getroottree().getroot()
 
@@ -154,14 +155,13 @@ class View(ModelSQL, ModelView):
                 for attr in ('states', 'domain', 'spell', 'colors'):
                     if element.get(attr):
                         try:
-                            value = safe_eval(element.get(attr), CONTEXT)
+                            value = PYSONDecoder().decode(element.get(attr))
                             validates.get(attr, lambda a: True)(value)
                         except Exception, e:
-                            logger = logging.getLogger('ir')
                             logger.error('Invalid pyson view element "%s:%s":'
-                                '\n%s\n%s'
-                                % (element.get('id') or element.get('name'),
-                                    attr, str(e), xml))
+                                '\n%s\n%s',
+                                element.get('id') or element.get('name'), attr,
+                                str(e), xml)
                             return False
                 for child in element:
                     if not encode(child):
@@ -221,7 +221,7 @@ class ShowView(Wizard):
         def __init__(self, model_name, buttons):
             StateView.__init__(self, model_name, None, buttons)
 
-        def get_view(self):
+        def get_view(self, wizard, state_name):
             pool = Pool()
             View = pool.get('ir.ui.view')
             view = View(Transaction().context.get('active_id'))
@@ -339,6 +339,8 @@ class ViewTreeState(ModelSQL, ModelView):
 
     @classmethod
     def set(cls, model, domain, child_name, nodes, selected_nodes):
+        # Normalize the json domain
+        domain = json.dumps(json.loads(domain))
         current_user = Transaction().user
         records = cls.search([
                 ('user', '=', current_user),
