@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 from itertools import chain
 from sql import Cast, Literal
 from sql.functions import Substring, Position
@@ -218,7 +218,7 @@ class One2Many(Field):
                 return Target(**data)
             else:
                 return Target(data)
-        value = [instance(x) for x in (value or [])]
+        value = tuple(instance(x) for x in (value or []))
         super(One2Many, self).__set__(inst, value)
 
     def convert_domain(self, domain, tables, Model):
