@@ -1,8 +1,8 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
-
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import copy
-from sql import Cast, Literal
+
+from sql import Cast, Literal, Null
 from sql.functions import Substring, Position
 from sql.conditionals import Case
 
@@ -82,7 +82,7 @@ class Property(Function):
         model_field = Field.__table__()
         model = IrModel.__table__()
 
-        #Fetch res ids that comply with the domain
+        # Fetch res ids that comply with the domain
         join = property_.join(model_field,
             condition=model_field.id == property_.field)
         join = join.join(model,
@@ -119,12 +119,12 @@ class Property(Function):
                 dom_operator = 'not in'
             return [('id', dom_operator, [x[0] for x in props])]
 
-        #Fetch the res ids that doesn't use the default value
+        # Fetch the res ids that doesn't use the default value
         cursor.execute(*property_.select(
                 Cast(Substring(property_.res,
                         Position(',', property_.res) + Literal(1)),
                     Model.id.sql_type().base),
-                where=property_cond & (property_.res != None)))
+                where=property_cond & (property_.res != Null)))
 
         fetchall = cursor.fetchall()
         if not fetchall:
@@ -165,7 +165,7 @@ class Property(Function):
             return column.in_(value)
         elif ((value is False or value is None)
                 and operator in ('=', '!=')):
-            return (column == None) == value
+            return (column == Null) == value
         elif operator == 'not like':
             return column.like(value)
         elif operator == 'not ilike':
