@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import os
 import sys
 import itertools
@@ -19,6 +19,8 @@ from trytond.transaction import Transaction
 from trytond.cache import Cache
 import trytond.convert as convert
 
+logger = logging.getLogger(__name__)
+
 ir_module = Table('ir_module_module')
 ir_model_data = Table('ir_model_data')
 
@@ -165,7 +167,7 @@ def create_graph(module_list):
         # add 'package' in the graph
         all_deps = deps + [x for x in xdep if x in all_packages]
         if reduce(lambda x, y: x and y in graph, all_deps, True):
-            if not package in current:
+            if package not in current:
                 packages.pop(0)
                 continue
             later.clear()
@@ -202,7 +204,6 @@ def load_module_graph(graph, pool, update=None, lang=None):
         update = []
     modules_todo = []
     models_to_update_history = set()
-    logger = logging.getLogger('modules')
     cursor = Transaction().cursor
 
     modules = [x.name for x in graph]
@@ -222,25 +223,25 @@ def load_module_graph(graph, pool, update=None, lang=None):
             if package_state not in ('to install', 'to upgrade'):
                 if package_state == 'installed':
                     package_state = 'to upgrade'
-                else:
+                elif package_state != 'to remove':
                     package_state = 'to install'
             for child in package.childs:
                 module2state[child.name] = package_state
             for type in classes.keys():
                 for cls in classes[type]:
-                    logger.info('%s:register %s' % (module, cls.__name__))
+                    logger.info('%s:register %s', module, cls.__name__)
                     cls.__register__(module)
             for model in classes['model']:
                 if hasattr(model, '_history'):
                     models_to_update_history.add(model.__name__)
 
-            #Instanciate a new parser for the package:
+            # Instanciate a new parser for the package:
             tryton_parser = convert.TrytondXmlHandler(pool=pool, module=module,
                 module_state=package_state)
 
             for filename in package.info.get('xml', []):
                 filename = filename.replace('/', os.sep)
-                logger.info('%s:loading %s' % (module, filename))
+                logger.info('%s:loading %s', module, filename)
                 # Feed the parser with xml content:
                 with tools.file_open(OPJ(module, filename)) as fp:
                     tryton_parser.parse_xmlstream(fp)
@@ -253,11 +254,13 @@ def load_module_graph(graph, pool, update=None, lang=None):
                 lang2 = os.path.splitext(os.path.basename(filename))[0]
                 if lang2 not in lang:
                     continue
-                logger.info('%s:loading %s' % (module,
-                        filename[len(package.info['directory']) + 1:]))
+                logger.info('%s:loading %s', module,
+                    filename[len(package.info['directory']) + 1:])
                 Translation = pool.get('ir.translation')
                 Translation.translation_import(lang2, module, filename)
 
+            if package_state == 'to remove':
+                continue
             cursor.execute(*ir_module.select(ir_module.id,
                     where=(ir_module.name == package.name)))
             try:
@@ -276,7 +279,7 @@ def load_module_graph(graph, pool, update=None, lang=None):
     for model_name in models_to_update_history:
         model = pool.get(model_name)
         if model._history:
-            logger.info('history:update %s' % model.__name__)
+            logger.info('history:update %s', model.__name__)
             model._update_history_table()
 
     # Vacuum :
@@ -293,6 +296,8 @@ def get_module_list():
         for file in os.listdir(MODULES_PATH):
             if file.startswith('.'):
                 continue
+            if file == '__pycache__':
+                continue
             if os.path.isdir(OPJ(MODULES_PATH, file)):
                 module_list.add(file)
     update_egg_modules()
@@ -316,11 +321,10 @@ def register_classes():
     trytond.webdav.register()
     import trytond.tests
     trytond.tests.register()
-    logger = logging.getLogger('modules')
 
     for package in create_graph(get_module_list())[0]:
         module = package.name
-        logger.info('%s:registering classes' % module)
+        logger.info('%s:registering classes', module)
 
         if module in ('ir', 'res', 'webdav', 'tests'):
             MODULES.append(module)
@@ -364,9 +368,6 @@ def load_modules(database_name, pool, update=None, lang=None):
         global res
         cursor = Transaction().cursor
         if update:
-            # Migration from 2.2: workflow module removed
-            cursor.execute(*ir_module.delete(
-                    where=(ir_module.name == 'workflow')))
             cursor.execute(*ir_module.select(ir_module.name,
                     where=ir_module.state.in_(('installed', 'to install',
                             'to upgrade', 'to remove'))))
@@ -391,7 +392,7 @@ def load_modules(database_name, pool, update=None, lang=None):
             fetchall = cursor.fetchall()
             if fetchall:
                 for (mod_name,) in fetchall:
-                    #TODO check if ressource not updated by the user
+                    # TODO check if ressource not updated by the user
                     cursor.execute(*ir_model_data.select(ir_model_data.model,
                             ir_model_data.db_id,
                             where=(ir_model_data.module == mod_name),
