@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 from threading import RLock
 import logging
 from trytond.modules import load_modules, register_classes
@@ -8,13 +8,18 @@ import __builtin__
 
 __all__ = ['Pool', 'PoolMeta', 'PoolBase']
 
+logger = logging.getLogger(__name__)
+
 
 class PoolMeta(type):
 
     def __new__(cls, name, bases, dct):
         new = type.__new__(cls, name, bases, dct)
         if '__name__' in dct:
-            new.__name__ = dct['__name__']
+            try:
+                new.__name__ = dct['__name__']
+            except TypeError:
+                new.__name__ = dct['__name__'].encode('utf-8')
         return new
 
 
@@ -134,7 +139,6 @@ class Pool(object):
         Set update to proceed to update
         lang is a list of language code to be updated
         '''
-        logger = logging.getLogger('pool')
         with self._lock:
             if not self._started:
                 self.start()
@@ -142,9 +146,9 @@ class Pool(object):
             # Don't reset pool if already init and not to update
             if not update and self._pool.get(self.database_name):
                 return
-            logger.info('init pool for "%s"' % self.database_name)
+            logger.info('init pool for "%s"', self.database_name)
             self._pool.setdefault(self.database_name, {})
-            #Clean the _pool before loading modules
+            # Clean the _pool before loading modules
             for type in self.classes.keys():
                 self._pool[self.database_name][type] = {}
             restart = not load_modules(self.database_name, self, update=update,
@@ -183,20 +187,6 @@ class Pool(object):
         with self._locks[self.database_name]:
             self._pool[self.database_name][type][cls.__name__] = cls
 
-    def object_name_list(self, type='model'):
-        '''
-        Return the object name list of a type
-
-        :param type: the type
-        :return: a list of name
-        '''
-        if type == '*':
-            res = []
-            for type in self.classes.keys():
-                res += self._pool[self.database_name][type].keys()
-            return res
-        return self._pool[self.database_name][type].keys()
-
     def iterobject(self, type='model'):
         '''
         Return an iterator over object name, object
