@@ -1,9 +1,9 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 
 import datetime
 from decimal import Decimal
-from trytond.model import ModelSQL, fields
+from trytond.model import ModelSQL, DictSchemaMixin, fields
 from trytond.pyson import Eval
 
 __all__ = [
@@ -16,6 +16,7 @@ __all__ = [
     'Date', 'DateDefault', 'DateRequired',
     'DateTime', 'DateTimeDefault', 'DateTimeRequired', 'DateTimeFormat',
     'Time', 'TimeDefault', 'TimeRequired', 'TimeFormat',
+    'TimeDelta', 'TimeDeltaDefault', 'TimeDeltaRequired',
     'One2One', 'One2OneTarget', 'One2OneRelation', 'One2OneRequired',
     'One2OneRequiredRelation', 'One2OneDomain', 'One2OneDomainRelation',
     'One2Many', 'One2ManyTarget',
@@ -32,9 +33,9 @@ __all__ = [
     'Reference', 'ReferenceTarget', 'ReferenceRequired',
     'Property',
     'Selection', 'SelectionRequired',
-    'Dict', 'DictDefault', 'DictRequired',
+    'DictSchema', 'Dict', 'DictDefault', 'DictRequired',
     'Binary', 'BinaryDefault', 'BinaryRequired',
-    'Many2OneDomainValidation', 'Many2OneTarget',
+    'Many2OneDomainValidation', 'Many2OneTarget', 'Many2OneOrderBy',
     ]
 
 
@@ -315,6 +316,31 @@ class TimeFormat(ModelSQL):
     time = fields.Time(string='Time', format='%H:%M')
 
 
+class TimeDelta(ModelSQL):
+    'TimeDelta'
+    __name__ = 'test.timedelta'
+    timedelta = fields.TimeDelta(string='TimeDelta', help='Test timedelta',
+        required=False)
+
+
+class TimeDeltaDefault(ModelSQL):
+    'TimeDelta Default'
+    __name__ = 'test.timedelta_default'
+    timedelta = fields.TimeDelta(string='TimeDelta', help='Test timedelta',
+        required=False)
+
+    @staticmethod
+    def default_timedelta():
+        return datetime.timedelta(seconds=3600)
+
+
+class TimeDeltaRequired(ModelSQL):
+    'TimeDelta Required'
+    __name__ = 'test.timedelta_required'
+    timedelta = fields.TimeDelta(string='TimeDelta', help='Test timedelta',
+        required=True)
+
+
 class One2One(ModelSQL):
     'One2One'
     __name__ = 'test.one2one'
@@ -629,10 +655,17 @@ class SelectionRequired(ModelSQL):
         'Selection', required=True)
 
 
+class DictSchema(DictSchemaMixin, ModelSQL):
+    'Dict Schema'
+    __name__ = 'test.dict.schema'
+
+
 class Dict(ModelSQL):
     'Dict'
     __name__ = 'test.dict'
-    dico = fields.Dict(None, 'Test Dict')
+    dico = fields.Dict('test.dict.schema', 'Test Dict')
+    dico_string = dico.translated('dico')
+    dico_string_keys = dico.translated('dico', 'keys')
 
 
 class DictDefault(ModelSQL):
@@ -664,7 +697,7 @@ class BinaryDefault(ModelSQL):
 
     @staticmethod
     def default_binary():
-        return buffer('default')
+        return b'default'
 
 
 class BinaryRequired(ModelSQL):
@@ -676,6 +709,7 @@ class BinaryRequired(ModelSQL):
 class Many2OneTarget(ModelSQL):
     "Many2One Domain Validation Target"
     __name__ = 'test.many2one_target'
+    _order_name = 'value'
 
     active = fields.Boolean('Active')
     value = fields.Integer('Value')
@@ -694,3 +728,9 @@ class Many2OneDomainValidation(ModelSQL):
             ('value', '>', 5),
             ])
     dummy = fields.Char('Dummy')
+
+
+class Many2OneOrderBy(ModelSQL):
+    "Many2One OrderBy"
+    __name__ = 'test.many2one_orderby'
+    many2one = fields.Many2One('test.many2one_target', 'many2one')
