@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import os
 import time
 import urllib
@@ -709,7 +709,7 @@ class Attachment(ModelSQL, ModelView):
         super(Attachment, cls).__setup__()
         cls._error_messages.update({
                 'collection_attachment_name': ('You can not create an '
-                    'attachment named "%(attachment)s in collection '
+                    'attachment named "%(attachment)s" in collection '
                     '"%(collection)s" because there is already a collection '
                     'with that name.')
                 })
